#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: calum
"""

from fileinput import filename
from netCDF4 import Dataset
from os import listdir, walk
from os.path import isfile, join
from datetime import datetime, timedelta
from time import perf_counter
import numpy as np
import pandas as pd
import xarray as xr
import argparse
import sys
import os
import math
import re

import dask.dataframe as dd

import geopandas as gpd
from geopandas import GeoDataFrame
from shapely.geometry import Polygon, Point


def dataframe_to_nc(df, filewrite):
    """
    Output data to netcdf
    """
    if "grid_cell" in df.columns:
        df = df.drop("grid_cell", axis=1)
    if "index_right" in df.columns:
        df = df.drop("index_right", axis=1)
    print("HEEEERRREEE")
    print(df)
    print(df.columns)

    ## Output df as netcdf
    df = df.rename(columns={"obslon":"rlon", "obslat":"rlat", "Longitude":"lon", "Latitude":"lat", "longitude":"lon", "latitude":"lat"})
    df = df.drop(columns=["obstype", "obsn", "obsid", "obslev", "lon_cen", "lat_cen"])
    if "AOD550_uncertainty" in df.columns:
        df = df.drop(columns=["AOD550_uncertainty"])
    ds = xr.Dataset.from_dataframe(df)

    ds.to_netcdf("{}.nc".format(filewrite))


def dataframe_to_nc_full(df, filewrite):
    #Output data to netcdf
    
    if "grid_cell" in df.columns:
        df = df.drop("grid_cell", axis=1)
    if "index_right" in df.columns:
        df = df.drop("index_right", axis=1)
    print("Output df")
    print(df)

    rlon = np.array(list(set(df["obslon"].values)))
    rlat = np.array(list(set(df["obslat"].values)))

    rlon = np.sort(rlon)
    rlat = np.sort(rlat)

    rlon_dict = {}
    for n, val in enumerate(rlon):
        rlon_dict[val] = n
    
    rlat_dict = {}
    for n, val in enumerate(rlat):
        rlat_dict[val] = n

    obs = np.empty((len(rlat_dict), len(rlon_dict)),)
    obserr = np.empty((len(rlat_dict), len(rlon_dict)),)
    time = np.empty((len(rlat_dict), len(rlon_dict)),)
    lat = np.empty((len(rlat_dict), len(rlon_dict)),)
    lon = np.empty((len(rlat_dict), len(rlon_dict)),)

    time[:] = np.nan

    obs[:] = np.nan
    obserr[:] = np.nan
    lat[:] = np.nan
    lon[:] = np.nan

    epoch_time = datetime(1970, 1, 1)

    for i in df.index:
        rlon_val = df.loc[i, "obslon"]
        rlat_val = df.loc[i, "obslat"]
        obs_val = df.loc[i, "obs"]
        obserr_val = df.loc[i, "obserr"]
        lon_val = df.loc[i, "Longitude"]
        lat_val = df.loc[i, "Latitude"]
        #time_val = df.loc[i, "time"]
        index_lon = rlon_dict[rlon_val]
        index_lat = rlat_dict[rlat_val]
        obs[index_lat, index_lon] = obs_val
        obserr[index_lat, index_lon] = obserr_val
        lon[index_lat, index_lon] = lon_val
        lat[index_lat, index_lon] = lat_val
        #time[index_lat, index_lon] = int((time_val - epoch_time).total_seconds())

    print(time)

    ## Output df as netcdf
    ds = xr.Dataset(
        data_vars=dict(
            obs=(["rlat", "rlon"], obs),
            obserr=(["rlat", "rlon"], obserr),
            lon=(["rlat", "rlon"], lon),
            lat=(["rlat", "rlon"], lat)
            #time=(["lat", "lon"], time),
            
        ),
        coords=dict(
            rlat=(["rlat"], rlat),
            rlon=(["rlon"], rlon),
        ),
        attrs=dict(description="Monarch da preprocessed data"),#Pass the grid refrences here
    )

    #ds["time"].attrs = {}

    ds.to_netcdf("{}.nc".format(filewrite))






def frequency_int(time_interval):
    if time_interval == "H":
        frequency = 1
    else:
        frequency = int(time_interval[0])
    return frequency


def prepare_obs(df):
    
    """
    group observation files by type, lon and lat and check if there are more than 1 
    value for each lon lat 
    
    obstype, obslon, obslat obsn will be 1d arrays
    obsid, obslev, obs and obserr will be matrixes with 0 value where we dont have 
    more than 1 obs
        observation = dataFrame with obsHeader
    """
    # Converting all columns that are needed for the output to numpy arrays
    # May have to create arrays of zeros and then fill them like done in the previous prepare_obs
    obstype_in = df["obstype"].to_numpy()
    obslon_in = df["obslon"].to_numpy()
    obslat_in = df["obslat"].to_numpy()
    obsn_in = df["obsn"].to_numpy()

    #df = df.drop_duplicates()
    gb = df.groupby(['obstype','obslon','obslat'])
    nentries = len(gb.groups)
    
    #maxobs = max(obsn_in)
    maxobs = 1

    obsid_in = df["obsid"].to_numpy()
    obsid_in = np.resize(obsid_in, (nentries, maxobs))
    obslev_in  = df["obslev"].to_numpy()
    obslev_in = np.resize(obslev_in, (nentries, maxobs))
    obs_in = df['obs'].to_numpy()
    obs_in = np.resize(obs_in, (nentries, maxobs))
    obserr_in = df["obserr"].to_numpy()
    obserr_in = np.resize(obserr_in, (nentries, maxobs))


    return obstype_in, obslon_in, obslat_in, obsn_in, obsid_in, obslev_in, obs_in, obserr_in, nentries, maxobs

#Function to introduce boolean arguments 
def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')
    

